//********************************************************
//        Image and 2D-data visulization and analysis   
//        Based on PixeLook library for Delphi 6
//                   (www.pixelook.com)
//        (c) PixeLook Development Group, 2002
//********************************************************

#ifndef __ZL_H
#define __ZL_H

#include <windows.h>

//---------------------------
// Constants for simple and advanced viewers
//---------------------------
#define VT_SIMPLE  0
#define VT_ADVANCED 1

//---------------------------
// Constants for data types
//---------------------------

// DT_NONE - empty image
#define DT_NONE 0

// DT_1BITIMAGE - standard Windows 1-bit image
#define DT_1BITIMAGE 14

// DT_4BITIMAGE - standard Windows 4-bit image
#define DT_4BITIMAGE 15

// DT_8BITIMAGE - standard Windows 8-bit image
#define DT_8BITIMAGE 1

// DT_15BITIMAGE - standard Windows 15-bit high color image with 0RRRRRGGGGGBBBBB-bit mask
#define DT_15BITIMAGE 12

// DT_16BITIMAGE - standard Windows 16-bit high color image with RRRRRGGGGGGBBBBB-bit mask
#define DT_16BITIMAGE 13

// DT_24BITIMAGE - standard Windows 24-bit color image
#define DT_24BITIMAGE 2

// DT_32BITIMAGE - standard Windows 32-bit color image
#define DT_32BITIMAGE 3

// DT_BYTE - multichannel (Depth = any number) byte image
#define DT_BYTE 4

// DT_WORD - multichannel (Depth = any number) 16-bit unsigned integer image
#define DT_WORD 5

// DT_SMALLINT - multichannel (Depth = any number) 16-bit signed integer image
#define DT_SMALLINT 6

// DT_INTEGER - multichannel (Depth = any number) 32-bit integer image
#define DT_INTEGER 7

// DT_SINGLE - multichannel (Depth = any number) 32-bit floating point image
#define DT_SINGLE 8

// DT_DOUBLE - multichannel (Depth = any number) 64-bit floating point image
#define DT_DOUBLE 9



//---------------------------
// Templates for all ZL.DLL functions
//---------------------------
typedef void* (* VCREATEFUNC)(int,HINSTANCE,int,int,int,int);
typedef void  (* VFREEFUNC)(void **);
typedef void* (* IMCREATEFUNC)(LPTSTR,int,int,int,int,void *,BOOL);
typedef void  (* IMSHOWFUNC)(void *,void *);
typedef void  (* IMSHOWMODALFUNC)(void *,int,int,int,int,int);
typedef void* (* IMFREEFUNC)(void *,void *);
typedef int   (* WAITKEYFUNC)(void);
typedef void  (* PROCMESFUNC)(void);
typedef void  (* SETAPPHANDLEPROC)(hwnd);

//---------------------------
// CZL struct wraps ZL.DLL and includes the handle of the DLL
// and all its functions.
//---------------------------

typedef struct {

//module handle of the DLL
	HMODULE ZLModule;

// VCreate - creates an external viewer.
// Syntax:
//           void* VCreate(int ViewerType, HINSTANCE WinInstance,
//                        int Left, int Top, int Width, int Height)
// Description:
//	         The viewer can be simple or advanced (see VT_...
//           constants below), which is defined by ViewerType.
//			 (Left,Top) determine position of the left-upper corner
//           of the viewer in the screen. Width, Height determine dimensions of the
//           viewer window
//           Returns pointer to the created viewer
	VCREATEFUNC VCreate;

// VFree - destroys a viewer created by VCreate
// Syntax:
//           void VFree (void **Viewer)
// Descriprion:
//           Pass a pointer to the viewer pointer created by VCreate as a parameter
//           to VFree. VFree destroys the viewer and sets its pointer to NULL
	VFREEFUNC VFree;

// ImCreate - creates an image in internal format from 2D-array
// Syntax:
//			void * ImCreate(LPSTR ImageName, int Width, int Height, int Depth,
//							int DataType, void *PixelStart, BOOL Reverse)
// Description:
//          ImageName - unique string for each image
//			Width, Height - image sizes in pixels
//			Depth - any number of channels
//			DataType - type of pixel data (see DT_... constants below)
//			PixelStart - pointer to pixel data
//			Reverse defines if the order of scan lines is from bottom to top (BMP-like)
//			Returns pointer to created image
	IMCREATEFUNC ImCreate;

// ImShow - associates an image with a viewer
// Syntax:
//			void ImShow(void *Viewer, void *Image)
// Description:
//			Image - is an image created by ImCreate
//			Viewer - is a viewer created by VCreate
	IMSHOWFUNC ImShow;

// ImShowModal - shows an image in a viewer. The viewer is created internally
//               then appears as a modal window, then is destroyed internally
// Syntax:
//			void ImShowModal(void *Image,int ViewerType,
//                        int Left, int Top, int Width, int Height)
// Description:
//			Image - is an image created by ImCreate
//                      See VCreate function for other parameters
	IMSHOWMODALFUNC ImShowModal;

// ImFree - destroys an image
// Syntax:
//			void ImFree(void *Viewer, void *Image)
// Description:
//          ImFree first hides and image in its viewer (if any), then destroys it
//			Image - is an image created by ImCreate
//			Viewer - is a viewer created by VCreate and associated vith the image
	IMFREEFUNC ImFree;

// WaitKey - waits a key from the keyboard and returns the key code
	WAITKEYFUNC WaitKey;

// ProcessMessages - processes all messages in message queue
	PROCMESFUNC ProcessMessages;

// SetAppHandle - sets application instance handle to all viewers to hide them
// from the Taskbar
	SETAPPHANDLEPROC SetAppHandle;// in current version YOU MAY NOT USE THIS
} CZL;

//---------------------------
// Function for loading ZL.DLL and filling CZL structure
//---------------------------
void LoadZL(CZL *ZL)
{
    ZL->ZLModule = LoadLibrary("ZL.DLL");

    ZL->VCreate = (VCREATEFUNC) GetProcAddress(ZL->ZLModule, "VCreate");
	ZL->VFree = (VFREEFUNC) GetProcAddress(ZL->ZLModule, "VFree");
	ZL->ImCreate = (IMCREATEFUNC) GetProcAddress(ZL->ZLModule, "ImCreate");
	ZL->ImShow = (IMSHOWFUNC) GetProcAddress(ZL->ZLModule, "ImShow");
	ZL->ImShowModal = (IMSHOWMODALFUNC) GetProcAddress(ZL->ZLModule, "ImShowModal");
	ZL->ImFree = (IMFREEFUNC) GetProcAddress(ZL->ZLModule, "ImFree");
	ZL->WaitKey = (WAITKEYFUNC) GetProcAddress(ZL->ZLModule, "WaitKey");
	ZL->ProcessMessages = (PROCMESFUNC) GetProcAddress(ZL->ZLModule, "ProcessMessages");
	ZL->SetAppHandle = (SETAPPHANDLEPROC) GetProcAddress(ZL->ZLModule, "SetAppHandle");
}

//---------------------------
// Function for unloading ZL.DLL
//---------------------------
void UnLoadZL(CZL *ZL)
{
	FreeLibrary(ZL->ZLModule);
}




//---------------------------
//----------- Multi-thread stuff ----------------
//Multithreading is useful in console application,
//because viewers are created in sepatate threads with their
//own message processing. This cause no problem with viewer
//redrawing. In normal Windows applications you can directly
//use functions from CZL
//---------------------------


//---------------------------
// Template functions for initialization and finalization of a thread
//---------------------------
typedef int  (* CALLBACK_ZLINIT) (LPVOID Param);
typedef void (* CALLBACK_ZLFIN) (LPVOID Param);


//---------------------------
// CZLThread structure is a wrapprer for threads
//---------------------------
typedef struct{
	CZL *ZL;  // Info about ZL.DLL
	DWORD ThreadId; //Thread identifier (returned by CreateThread)
	HANDLE HT;  // handle of the thread
	int Ready; // Indicates if a viewer is created and ready for visualization
	CALLBACK_ZLINIT Init; //Thread initialization callback function.
						  //Usually used for creation of viewer
	                      //Should return a non-zero value which becomes the
						  //Ready field of this structure
	CALLBACK_ZLFIN Finalize; //Thread finalization callback function.
						     //Usually used for viewer destroying
} CZLThread;


//---------------------------
//Thread function.
//Performs loading ZL.DLL, user-defined initialization of the thread,
//message processing, user-defined finalization of the thread,
//and unloading ZL.DLL
//---------------------------
DWORD WINAPI ZLThreadProc(LPVOID Param)
{
	MSG Msg;
	CZLThread *Th;

	Th = (CZLThread*) Param;
	LoadZL(Th->ZL);
	Th->Ready = Th->Init(Param);

	while (GetMessage(&Msg,0,0,0))
	{
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}

	Th->Finalize(Param);
	UnLoadZL(Th->ZL);
	return 0;
}

//---------------------------
// CreateZLThread creates a new thread,
// fills CZLThread and CZL structures, and passes user-defined callback functions
//
//---------------------------
void CreateZLThread(CZLThread *T, CZL *ZL, CALLBACK_ZLINIT Init, CALLBACK_ZLFIN Finalize)
{
	T->ZL = ZL;
	T->Ready = 0;
	T->Finalize = Finalize;
	T->Init = Init;
	T->HT = CreateThread(NULL,0,ZLThreadProc,T,0 ,&(T->ThreadId));
}



#endif
